/**********************************************************************/
/*                                                                    */
/*   Licensed Materials - Property of IBM.                            */
/*   5724-T42 5724-T43                                                */
/*   Copyright IBM Corp. 2007.                                        */
/*   All Rights Reserved.                                             */
/*   US Government Users Restricted Rights -                          */
/*   Use, duplication or disclosure restricted by                     */
/*   GSA ADP Schedule Contract with IBM Corp.                         */
/*                                                                    */
/**********************************************************************/


#if !defined(_OMP_H_)
#define	_OMP_H_
/* The #endif for this is at the end of the file. */

#ifdef __cplusplus
extern "C" {
#endif

//#include <pthread.h>

//it is for mutex_ea_t for both spu and ppu
typedef unsigned int omp_lock_t;
//typedef int omp_lock_t;
  
typedef struct {
  unsigned long long int lock;
  int       count;
  int      thread;
} omp_nest_lock_t;
  
double omp_get_wtick(void);
double omp_get_wtime(void);

#if defined(_OPENMP) 

void omp_set_num_threads(int num);
int  omp_get_num_threads(void);
int  omp_get_max_threads(void);
int  omp_get_thread_num(void);
int  omp_get_num_procs(void);
int  omp_in_parallel(void);

void omp_set_dynamic(int flag);
int  omp_get_dynamic(void);

void omp_set_nested(int flag);
int  omp_get_nested(void);

void omp_init_lock(omp_lock_t *lock);
void omp_init_nest_lock(omp_nest_lock_t *lock);

void omp_destroy_lock(omp_lock_t *lock);
void omp_destroy_nest_lock(omp_nest_lock_t *lock);

void omp_set_lock(omp_lock_t *lock);
void omp_set_nest_lock(omp_nest_lock_t *lock);

void omp_unset_lock(omp_lock_t *lock);
void omp_unset_nest_lock(omp_nest_lock_t *lock);

int  omp_test_lock(omp_lock_t *lock);
int  omp_test_nest_lock(omp_nest_lock_t *lock);

void omp_set_debug_level(int);

#else 

#if __cplusplus
#define __omp_inline  static inline
#else
#define __omp_inline  static __inline
#endif

__omp_inline void omp_set_num_threads(int num) {
}

__omp_inline int omp_get_num_threads(void) {
  return 1;
}

__omp_inline int omp_get_max_threads(void) {
  return 1;
}

__omp_inline int omp_get_thread_num(void) {
  return 0;
}

__omp_inline int omp_get_num_procs(void) {
  return 1;
}

__omp_inline int omp_in_parallel(void) {
  return 0;
}

__omp_inline void omp_set_dynamic(int flag) {
}

__omp_inline int omp_get_dynamic(void) {
  return 0;
}

__omp_inline void omp_set_nested(int flag) {
}

__omp_inline int omp_get_nested(void) {
  return 0;
}

__omp_inline void omp_init_lock(omp_lock_t  *lock) {
}

__omp_inline void omp_init_nest_lock(omp_nest_lock_t *lock) {
  lock->count = 0;
}

__omp_inline void omp_destroy_lock(omp_lock_t *lock) {
}

__omp_inline void omp_destroy_nest_lock(omp_nest_lock_t *lock) {
}

__omp_inline void omp_set_lock(omp_lock_t *lock) {
}

__omp_inline void omp_set_nest_lock(omp_nest_lock_t *lock) {
  lock->count++;
}

__omp_inline void omp_unset_lock(omp_lock_t *lock) {
}

__omp_inline void omp_unset_nest_lock(omp_nest_lock_t *lock) {
  lock->count--;
}

__omp_inline int omp_test_lock(omp_lock_t *lock) {
  return 1;
}

__omp_inline int omp_test_nest_lock(omp_nest_lock_t *lock) {
  lock->count++;
  return lock->count;
}

#undef __omp_inline

#endif  /* (_OPENMP) */ 


#ifdef __cplusplus
}
#endif

#endif

